@@ -26,9 +26,9 @@
 #include <errno.h>
 
 #include "kerncompat.h"
-#include "ctree.h"
+#include "kernel-shared/ctree.h"
 #include "ioctl.h"
-#include "volumes.h"
+#include "kernel-shared/volumes.h"
 
 #include "cmds/commands.h"
 #include "common/utils.h"
